@@ -383,6 +383,7 @@ dzl_multi_paned_destroy_child_handle (DzlMultiPaned      *self,
 
   if (child->handle != NULL)
     {
+      gtk_widget_unregister_window (GTK_WIDGET (self), child->handle);
       gdk_window_destroy (child->handle);
       child->handle = NULL;
     }
@@ -1984,7 +1985,7 @@ dzl_multi_paned_finalize (GObject *object)
 
   g_assert (priv->children->len == 0);
 
-  dzl_clear_pointer (&priv->children, g_array_unref);
+  g_clear_pointer (&priv->children, g_array_unref);
   g_clear_object (&priv->gesture);
 
   G_OBJECT_CLASS (dzl_multi_paned_parent_class)->finalize (object);
